@@ -1,6 +1,6 @@
 /*
    Copyright (c) 2000, 2012, Oracle and/or its affiliates.
-   Copyright (c) 2019, MariaDB Corporation.
+   Copyright (c) 2019, 2021, MariaDB Corporation.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -66,7 +66,7 @@ extern "C" {
 #endif
 
 #ifdef DBUG_OFF
-#if defined(HAVE_STPCPY) && MY_GNUC_PREREQ(3, 4) && !defined(__INTEL_COMPILER)
+#if defined(HAVE_STPCPY) && defined(__GNUC__) && !defined(__INTEL_COMPILER)
 #define strmov(A,B) __builtin_stpcpy((A),(B))
 #elif defined(HAVE_STPCPY)
 #define strmov(A,B) stpcpy((A),(B))
@@ -199,9 +199,22 @@ extern ulonglong strtoull(const char *str, char **ptr, int base);
 
 #include <mysql/plugin.h>
 
-#define STRING_WITH_LEN(X) (X), ((size_t) (sizeof(X) - 1))
-#define USTRING_WITH_LEN(X) ((uchar*) X), ((size_t) (sizeof(X) - 1))
-#define C_STRING_WITH_LEN(X) ((char *) (X)), ((size_t) (sizeof(X) - 1))
+#ifdef __cplusplus
+#include <type_traits>
+template<typename T> inline constexpr const char *_swl_check(T s)
+{
+  static_assert(std::is_same<T, const char (&)[sizeof(T)]>::value
+             || std::is_same<T, const char [sizeof(T)]>::value,
+             "Wrong argument for STRING_WITH_LEN()");
+  return s;
+}
+#define STRING_WITH_LEN(X) _swl_check<decltype(X)>(X), ((size_t) (sizeof(X) - 1))
+#else
+#define STRING_WITH_LEN(X) (X ""), ((size_t) (sizeof(X) - 1))
+#endif
+
+#define USTRING_WITH_LEN(X) (uchar*) STRING_WITH_LEN(X)
+#define C_STRING_WITH_LEN(X) (char *) STRING_WITH_LEN(X)
 #define LEX_STRING_WITH_LEN(X) (X).str, (X).length
 
 typedef struct st_mysql_const_lex_string LEX_CSTRING;
@@ -226,42 +239,76 @@ static inline void lex_string_set3(LEX_CSTRING *lex_str, const char *c_str,
   lex_str->length= len;
 }
 
-/*
-  Copies src into dst and ensures dst is a NULL terminated C string.
+/**
+  Copies a string.
 
-  Returns 1 if the src string was truncated due to too small size of dst.
-  Returns 0 if src completely fit within dst. Pads the remaining dst with '\0'
-
-  Note: dst_size must be > 0
+  @param dst        destination buffer, will be NUL padded.
+  @param dst_size   size of dst buffer, must be > 0
+  @param src        NUL terminated source string
 */
-static inline int safe_strcpy(char *dst, size_t dst_size, const char *src)
+static inline void safe_strcpy(char *dst, size_t dst_size, const char *src)
 {
-  memset(dst, '\0', dst_size);
-  strncpy(dst, src, dst_size - 1);
-  /*
-     If the first condition is true, we are guaranteed to have src length
-     >= (dst_size - 1), hence safe to access src[dst_size - 1].
-  */
-  if (dst[dst_size - 2] != '\0' && src[dst_size - 1] != '\0')
-    return 1; /* Truncation of src. */
+  DBUG_ASSERT(dst_size > 0);
+
+  /* 1) IF there is a 0 byte in the first dst_size bytes of src, strncpy will
+   *    0-terminate dst, and pad dst with additional 0 bytes out to dst_size.
+   *
+   * 2) IF there is no 0 byte in the first dst_size bytes of src, strncpy will
+   *    copy dst_size bytes, and the final byte won't be 0.
+   *
+   * In GCC 8+, the `-Wstringop-truncation` warning may object to strncpy()
+   * being used in this way, so we need to disable this warning for this
+   * single statement.
+   */
+
+#if defined __GNUC__ && __GNUC__ >= 8
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wstringop-truncation"
+#endif
+  strncpy(dst, src, dst_size);
+#if defined __GNUC__ && __GNUC__ >= 8
+#pragma GCC diagnostic pop
+#endif
+  dst[dst_size - 1]= 0;
+}
+
+/**
+  Copies a string, checking for truncation.
+
+  @param dst        destination buffer, will be NUL padded.
+  @param dst_size   size of dst buffer, must be > 0
+  @param src        NUL terminated source string
+
+  @retval 1 if the src string was truncated due to too small size of dst.
+  @retval 0 if src completely fit within dst,
+*/
+static inline int safe_strcpy_truncated(char *dst, size_t dst_size,
+                                        const char *src)
+{
+  DBUG_ASSERT(dst_size > 0);
+
+  strncpy(dst, src, dst_size);
+  if (dst[dst_size - 1])
+  {
+    dst[dst_size - 1]= 0;
+    return 1;
+  }
   return 0;
 }
 
-/*
-  Appends src to dst and ensures dst is a NULL terminated C string.
+/**
+  Appends src to dst and ensures dst is a NUL terminated C string.
 
-  Returns 1 if the src string was truncated due to too small size of dst.
-  Returns 0 if src completely fit within the remaining dst space. Pads the
-            remaining dst with '\0'.
-
-  Note: dst_size must be > 0
+  @retval 1 if the src string was truncated due to too small size of dst.
+  @retval 0 if src completely fit within the remaining dst space,
+  including NUL termination.
 */
 static inline int safe_strcat(char *dst, size_t dst_size, const char *src)
 {
   size_t init_len= strlen(dst);
-  if (unlikely(init_len >= dst_size - 1))
+  if (init_len > dst_size)
     return 1;
-  return safe_strcpy(dst + init_len, dst_size - init_len, src);
+  return safe_strcpy_truncated(dst + init_len, dst_size - init_len, src);
 }
 
 #ifdef __cplusplus
