@@ -866,12 +866,12 @@ void end_master_info(Master_info* mi)
 }
 
 /* Multi-Master By P.Linux */
-const uchar *get_key_master_info(const void *mi_, size_t *length, my_bool)
+uchar *get_key_master_info(Master_info *mi, size_t *length,
+                           my_bool not_used __attribute__((unused)))
 {
-  auto mi= static_cast<const Master_info *>(mi_);
   /* Return lower case name */
   *length= mi->cmp_connection_name.length;
-  return reinterpret_cast<const uchar *>(mi->cmp_connection_name.str);
+  return (uchar*) mi->cmp_connection_name.str;
 }
 
 /*
@@ -881,9 +881,8 @@ const uchar *get_key_master_info(const void *mi_, size_t *length, my_bool)
   Stops associated slave threads and frees master_info
 */
 
-void free_key_master_info(void *mi_)
+void free_key_master_info(Master_info *mi)
 {
-  Master_info *mi= static_cast<Master_info*>(mi_);
   DBUG_ENTER("free_key_master_info");
   mysql_mutex_unlock(&LOCK_active_mi);
 
@@ -1120,9 +1119,10 @@ bool Master_info_index::init_all_master_info()
   }
 
   /* Initialize Master_info Hash Table */
-  if (my_hash_init(PSI_INSTRUMENT_ME, &master_info_hash, system_charset_info,
-                   MAX_REPLICATION_THREAD, 0, 0, get_key_master_info,
-                   free_key_master_info, HASH_UNIQUE))
+  if (my_hash_init(PSI_INSTRUMENT_ME, &master_info_hash, system_charset_info, 
+                   MAX_REPLICATION_THREAD, 0, 0, 
+                   (my_hash_get_key) get_key_master_info, 
+                   (my_hash_free_key)free_key_master_info, HASH_UNIQUE))
   {                                                      
     sql_print_error("Initializing Master_info hash table failed");
     DBUG_RETURN(1);
