@@ -214,8 +214,9 @@ bool sys_var::update(THD *thd, set_var *var)
 
     /*
       Make sure we don't session-track variables that are not actually
-      part of the session. tx_isolation and and tx_read_only for example
-      exist as GLOBAL, SESSION, and one-shot ("for next transaction only").
+      part of the session. transaction_isolation and transaction_read_only for
+      example exist as GLOBAL, SESSION, and one-shot ("for next transaction
+      only").
     */
     if ((var->type == OPT_SESSION) && (!ret))
     {
@@ -310,7 +311,7 @@ do {                                                \
     case SHOW_HA_ROWS:  do_num_val (ha_rows,CMD);
 
 #define case_for_double(CMD)                        \
-    case SHOW_DOUBLE:   do_num_val (double,CMD)
+    case SHOW_DOUBLE:   do_num_val (double,CMD);
 
 #define case_get_string_as_lex_string               \
     case SHOW_CHAR:                                 \
@@ -790,7 +791,6 @@ int set_var::check(THD *thd)
   if (!value)
     return 0;
 
-  thd->where= THD_WHERE::SET_LIST;
   if (value->fix_fields_if_needed_for_scalar(thd, &value))
     return -1;
   if (var->check_update_type(value))
@@ -897,7 +897,6 @@ int set_var_user::check(THD *thd)
     Item_func_set_user_var can't substitute something else on its place =>
     0 can be passed as last argument (reference on item)
   */
-  thd->where= THD_WHERE::SET_LIST;
   return (user_var_item->fix_fields(thd, (Item**) 0) ||
           user_var_item->check(0)) ? -1 : 0;
 }
@@ -1114,7 +1113,7 @@ int fill_sysvars(THD *thd, TABLE_LIST *tables, COND *cond)
     fields[0]->store(name_buffer, strlen(name_buffer), scs);
 
     if ((wild && wild_case_compare(system_charset_info, name_buffer, wild))
-        || (cond && !cond->val_bool()))
+        || (cond && !cond->val_int()))
       continue;
 
     mysql_mutex_lock(&LOCK_global_system_variables);
@@ -1552,4 +1551,3 @@ ulonglong get_system_variable_hash_version(void)
 {
   return system_variable_hash_version;
 }
-
