@@ -1,4 +1,4 @@
-/* Copyright (C) 2015-2025 Codership Oy <info@codership.com>
+/* Copyright (C) 2015-2023 Codership Oy <info@codership.com>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -31,8 +31,6 @@
 #include "wsrep_storage_service.h"
 #include "wsrep_thd.h"
 #include "wsrep_server_state.h"
-#include "log_event.h"
-#include "sql_class.h"
 
 #include <string>
 #include <sstream>
@@ -195,25 +193,6 @@ private:
   my_bool m_wsrep_ignore_table;
 };
 
-class wsrep_skip_locking
-{
-public:
-  wsrep_skip_locking(THD *thd)
-      : m_thd(thd)
-      , m_wsrep_skip_locking(thd->wsrep_skip_locking)
-  {
-    thd->wsrep_skip_locking= true;
-  }
-  ~wsrep_skip_locking()
-  {
-    m_thd->wsrep_skip_locking= m_wsrep_skip_locking;
-  }
-
-private:
-  THD *m_thd;
-  my_bool m_wsrep_skip_locking;
-};
-
 class thd_server_status
 {
 public:
@@ -343,7 +322,7 @@ static int open_table(THD *thd, const LEX_CSTRING *schema_name,
        (thd->get_stmt_da()->sql_errno() == ER_QUERY_INTERRUPTED));
 
   if (interrupted ||
-      !open_n_lock_single_table(thd, table_list, lock_type, flags))
+      !open_n_lock_single_table(thd, table_list, table_list->lock_type, flags))
   {
     close_thread_tables(thd);
     DBUG_RETURN(1);
@@ -750,6 +729,7 @@ int Wsrep_schema::init()
     WSREP_ERROR("Unable to get thd");
     DBUG_RETURN(1);
   }
+  thd->thread_stack= (char*)&thd;
   wsrep_init_thd_for_schema(thd);
 
   if (Wsrep_schema_impl::execute_SQL(thd, create_cluster_table_str.c_str(),
@@ -805,17 +785,11 @@ int Wsrep_schema::store_view(THD* thd, const Wsrep_view& view)
 #ifdef WSREP_SCHEMA_MEMBERS_HISTORY
   TABLE* members_history_table= 0;
 #endif /* WSREP_SCHEMA_MEMBERS_HISTORY */
-  Query_tables_list query_tables_list_backup;
 
   Wsrep_schema_impl::wsrep_off wsrep_off(thd);
   Wsrep_schema_impl::binlog_off binlog_off(thd);
   Wsrep_schema_impl::sql_safe_updates sql_safe_updates(thd);
 
-  /*
-    Backup and restore the query table list changes.
-  */
-  thd->lex->reset_n_backup_query_tables_list(&query_tables_list_backup);
-
   if (trans_begin(thd, MYSQL_START_TRANS_OPT_READ_WRITE))
   {
     WSREP_ERROR("Failed to start transaction for store view");
@@ -931,7 +905,6 @@ int Wsrep_schema::store_view(THD* thd, const Wsrep_view& view)
   thd->release_transactional_locks();
 
 out_not_started:
-  thd->lex->restore_backup_query_tables_list(&query_tables_list_backup);
   DBUG_RETURN(ret);
 }
 
@@ -956,17 +929,11 @@ Wsrep_view Wsrep_schema::restore_view(THD* thd, const Wsrep_id& own_id) const {
   int proto_ver= 0;
   wsrep_cap_t capabilities= 0;
   std::vector<Wsrep_view::member> members;
-  Query_tables_list query_tables_list_backup;
 
   // we don't want causal waits for reading non-replicated private data
   int const wsrep_sync_wait_saved= thd->variables.wsrep_sync_wait;
   thd->variables.wsrep_sync_wait= 0;
 
-  /*
-    Backup and restore the query table list changes.
-  */
-  thd->lex->reset_n_backup_query_tables_list(&query_tables_list_backup);
-
   if (trans_begin(thd, MYSQL_START_TRANS_OPT_READ_ONLY)) {
     WSREP_ERROR("wsrep_schema::restore_view(): Failed to start transaction");
     goto out;
@@ -1081,14 +1048,12 @@ Wsrep_view Wsrep_schema::restore_view(THD* thd, const Wsrep_id& own_id) const {
       os << "Restored cluster view:\n" << ret_view;
       WSREP_INFO("%s", os.str().c_str());
     }
-    thd->lex->restore_backup_query_tables_list(&query_tables_list_backup);
     DBUG_RETURN(ret_view);
   }
   else
   {
     WSREP_ERROR("wsrep_schema::restore_view() failed.");
     Wsrep_view ret_view;
-    thd->lex->restore_backup_query_tables_list(&query_tables_list_backup);
     DBUG_RETURN(ret_view);
   }
 }
@@ -1294,7 +1259,6 @@ int Wsrep_schema::remove_fragments(THD* thd,
   Wsrep_schema_impl::wsrep_ignore_table wsrep_ignore_table(thd);
   Wsrep_schema_impl::binlog_off binlog_off(thd);
   Wsrep_schema_impl::sql_safe_updates sql_safe_updates(thd);
-  Wsrep_schema_impl::wsrep_skip_locking skip_locking(thd);
 
   Query_tables_list query_tables_list_backup;
   Open_tables_backup open_tables_backup;
@@ -1475,7 +1439,6 @@ int Wsrep_schema::replay_transaction(THD* orig_thd,
 {
   DBUG_ENTER("Wsrep_schema::replay_transaction");
   DBUG_ASSERT(!fragments.empty());
-  Query_tables_list query_tables_list_backup;
 
   THD *thd= new THD(next_thread_id(), true);
   if (!thd)
@@ -1487,13 +1450,7 @@ int Wsrep_schema::replay_transaction(THD* orig_thd,
   thd->thread_stack= (orig_thd ? orig_thd->thread_stack : (char *) &thd);
   wsrep_assign_from_threadvars(thd);
 
-
-  /*
-    Backup and restore the query table list changes.
-  */
-  orig_thd->lex->reset_n_backup_query_tables_list(&query_tables_list_backup);
   int ret= ::replay_transaction(thd, orig_thd, rli, ws_meta, fragments);
-  orig_thd->lex->restore_backup_query_tables_list(&query_tables_list_backup);
 
   delete thd;
   DBUG_RETURN(ret);
@@ -1676,64 +1633,6 @@ int Wsrep_schema::recover_sr_transactions(THD *orig_thd)
   DBUG_RETURN(ret);
 }
 
-int Wsrep_schema::store_gtid_event(THD* thd,
-                                   const Gtid_log_event *gtid)
-{
-  DBUG_ENTER("Wsrep_schema::store_gtid_event");
-  int error=0;
-  void *hton= NULL;
-  const bool in_transaction= (gtid->flags2 & Gtid_log_event::FL_TRANSACTIONAL);
-  const bool in_ddl= (gtid->flags2 & Gtid_log_event::FL_DDL);
-
-  DBUG_PRINT("info", ("thd: %p, in_transaction: %d, in_ddl: %d "
-                      "in_active_multi_stmt_transaction: %d",
-                      thd, in_transaction, in_ddl,
-                      thd->in_active_multi_stmt_transaction()));
-
-  Wsrep_schema_impl::wsrep_ignore_table  ignore_table(thd);
-  Wsrep_schema_impl::binlog_off binlog_off(thd);
-  Wsrep_schema_impl::sql_safe_updates sql_safe_updates(thd);
-
-  rpl_group_info *rgi= thd->wsrep_rgi;
-  const uint64 sub_id= rpl_global_gtid_slave_state->next_sub_id(gtid->domain_id);
-  rpl_gtid current_gtid;
-  current_gtid.domain_id= gtid->domain_id;
-  current_gtid.server_id= gtid->server_id;
-  current_gtid.seq_no= gtid->seq_no;
-  rgi->gtid_pending= false;
-
-  DBUG_ASSERT(!in_transaction || thd->in_active_multi_stmt_transaction());
-
-  if ((error= rpl_global_gtid_slave_state->record_gtid(thd, &current_gtid,
-                                                      sub_id,
-                                                      in_transaction, false, &hton)))
-    goto out;
-
-  rpl_global_gtid_slave_state->update_state_hash(sub_id, &current_gtid, hton, rgi);
-
-  if (in_ddl)
-  {
-    // Commit transaction if this GTID is part of DDL-clause because
-    // DDL causes implicit commit assuming there is no multi statement
-    // transaction ongoing.
-    if((error= trans_commit_stmt(thd)))
-      goto out;
-
-    (void)trans_commit(thd);
-  }
-
-out:
-  if (error)
-  {
-    WSREP_DEBUG("Wsrep_schema::store_gtid_event %llu-%llu-%llu failed error=%s (%d).",
-                gtid->domain_id, gtid->server_id, gtid->seq_no, strerror(error), error);
-    (void)trans_rollback_stmt(thd);
-    (void)trans_rollback(thd);
-  }
-
-  DBUG_RETURN(error);
-}
-
 void Wsrep_schema::clear_allowlist()
 {
   THD* thd= new THD(next_thread_id());
@@ -1796,7 +1695,6 @@ void Wsrep_schema::store_allowlist(std::vector<std::string>& ip_allowlist)
   TABLE* allowlist_table= 0;
   TABLE_LIST allowlist_table_l;
   int error;
-
   Wsrep_schema_impl::init_stmt(thd);
   if (Wsrep_schema_impl::open_for_write(thd, allowlist_table_str.c_str(),
                                         &allowlist_table_l))
@@ -1842,9 +1740,9 @@ static void *allowlist_check_thread(void *param)
   Allowlist_check_arg *arg= (Allowlist_check_arg *) param;
 
   my_thread_init();
-  THD *thd= new THD(0);
-  thd->thread_stack= (char *) thd;
-  wsrep_init_thd_for_schema(thd);
+  THD thd(0);
+  thd.thread_stack= (char *) &thd;
+  wsrep_init_thd_for_schema(&thd);
 
   int error;
   TABLE *allowlist_table= 0;
@@ -1858,8 +1756,8 @@ static void *allowlist_check_thread(void *param)
   /*
    * Read allowlist table
    */
-  Wsrep_schema_impl::init_stmt(thd);
-  if (Wsrep_schema_impl::open_for_read(thd, allowlist_table_str.c_str(),
+  Wsrep_schema_impl::init_stmt(&thd);
+  if (Wsrep_schema_impl::open_for_read(&thd, allowlist_table_str.c_str(),
                                        &allowlist_table_l) ||
       (allowlist_table= allowlist_table_l.table,
        Wsrep_schema_impl::init_for_scan(allowlist_table)))
@@ -1900,10 +1798,9 @@ static void *allowlist_check_thread(void *param)
   {
     goto out;
   }
-  Wsrep_schema_impl::finish_stmt(thd);
-  (void) trans_commit(thd);
+  Wsrep_schema_impl::finish_stmt(&thd);
+  (void) trans_commit(&thd);
 out:
-  delete thd;
   my_thread_end();
   arg->response = match_found_or_empty;
   return 0;
