@@ -59,27 +59,34 @@ Created 10/25/1995 Heikki Tuuri
 #include "bzlib.h"
 #include "snappy-c.h"
 
-ATTRIBUTE_COLD void fil_space_t::set_corrupted() const
+ATTRIBUTE_COLD bool fil_space_t::set_corrupted() const noexcept
 {
   if (!is_stopping() && !is_corrupted.test_and_set())
+  {
     sql_print_error("InnoDB: File '%s' is corrupted", chain.start->name);
+    return true;
+  }
+  return false;
 }
 
 /** Try to close a file to adhere to the innodb_open_files limit.
+@param ignore_space Ignore the tablespace which is acquired by caller
 @param print_info   whether to diagnose why a file cannot be closed
 @return whether a file was closed */
-bool fil_space_t::try_to_close(bool print_info)
+bool fil_space_t::try_to_close(fil_space_t *ignore_space, bool print_info)
 {
   mysql_mutex_assert_owner(&fil_system.mutex);
   for (fil_space_t &space : fil_system.space_list)
   {
+    if (&space == ignore_space)
+      continue;
     switch (space.purpose) {
     case FIL_TYPE_TEMPORARY:
       continue;
     case FIL_TYPE_IMPORT:
       break;
     case FIL_TYPE_TABLESPACE:
-      if (is_predefined_tablespace(space.id))
+      if (space.id == TRX_SYS_SPACE || srv_is_undo_tablespace(space.id))
         continue;
     }
 
@@ -324,7 +331,7 @@ fil_node_t* fil_space_t::add(const char* name, pfs_os_file_t handle,
 		clear_closing();
 		if (++fil_system.n_open >= srv_max_n_open_files) {
 			reacquire();
-			try_to_close(true);
+			try_to_close(this, true);
 			release();
 		}
 	}
@@ -332,11 +339,14 @@ fil_node_t* fil_space_t::add(const char* name, pfs_os_file_t handle,
 	return node;
 }
 
-__attribute__((warn_unused_result, nonnull))
+__attribute__((warn_unused_result, nonnull(1)))
 /** Open a tablespace file.
 @param node  data file
+@param page  first page of the tablespace, or nullptr
+@param no_lsn  whether to skip the FIL_PAGE_LSN check
 @return whether the file was successfully opened */
-static bool fil_node_open_file_low(fil_node_t *node)
+static bool fil_node_open_file_low(fil_node_t *node, const byte *page,
+                                   bool no_lsn)
 {
   ut_ad(!node->is_open());
   ut_ad(node->space->is_closing());
@@ -381,7 +391,7 @@ static bool fil_node_open_file_low(fil_node_t *node)
 
     /* The following call prints an error message */
     if (os_file_get_last_error(true) == EMFILE + 100 &&
-        fil_space_t::try_to_close(true))
+        fil_space_t::try_to_close(nullptr, true))
       continue;
 
     ib::warn() << "Cannot open '" << node->name << "'.";
@@ -392,7 +402,7 @@ static bool fil_node_open_file_low(fil_node_t *node)
   bool comp_algo_invalid = false;
 
   if (node->size);
-  else if (!node->read_page0() ||
+  else if (!node->read_page0(page, no_lsn) ||
             // validate compression algorithm for full crc32 format
             (node->space->full_crc32() &&
              (comp_algo_invalid = !fil_comp_algo_loaded(comp_algo))))
@@ -423,8 +433,10 @@ static bool fil_node_open_file_low(fil_node_t *node)
 
 /** Open a tablespace file.
 @param node  data file
+@param page  first page of the tablespace, or nullptr
+@param no_lsn  whether to skip the FIL_PAGE_LSN check
 @return whether the file was successfully opened */
-static bool fil_node_open_file(fil_node_t *node)
+static bool fil_node_open_file(fil_node_t *node, const byte *page, bool no_lsn)
 {
   mysql_mutex_assert_owner(&fil_system.mutex);
   ut_ad(!node->is_open());
@@ -439,7 +451,7 @@ static bool fil_node_open_file(fil_node_t *node)
 
   for (ulint count= 0; fil_system.n_open >= srv_max_n_open_files; count++)
   {
-    if (fil_space_t::try_to_close(count > 1))
+    if (fil_space_t::try_to_close(nullptr, count > 1))
       count= 0;
     else if (count >= 2)
     {
@@ -463,7 +475,7 @@ static bool fil_node_open_file(fil_node_t *node)
 
   /* The node can be opened beween releasing and acquiring fil_system.mutex
   in the above code */
-  return node->is_open() || fil_node_open_file_low(node);
+  return node->is_open() || fil_node_open_file_low(node, page, no_lsn);
 }
 
 /** Close the file handle. */
@@ -669,6 +681,19 @@ fil_space_extend_must_retry(
 	return false;
 }
 
+bool recv_sys_t::check_sys_truncate()
+{
+  mysql_mutex_assert_owner(&fil_system.mutex);
+  if (!truncated_sys_space.lsn)
+    return false;
+  if (fil_system.sys_space->size <= fil_system.sys_space->recv_size)
+  {
+    truncated_sys_space={0,0};
+    return false;
+  }
+  return true;
+}
+
 /** @return whether the file is usable for io() */
 ATTRIBUTE_COLD bool fil_space_t::prepare_acquired()
 {
@@ -678,13 +703,16 @@ ATTRIBUTE_COLD bool fil_space_t::prepare_acquired()
   ut_ad(!id || purpose == FIL_TYPE_TEMPORARY ||
         node == UT_LIST_GET_FIRST(chain));
 
-  const bool is_open= node && (node->is_open() || fil_node_open_file(node));
+  const bool is_open= node &&
+    (node->is_open() || fil_node_open_file(node, nullptr, false));
 
   if (!is_open)
     release();
   else if (node->deferred);
   else if (auto desired_size= recv_size)
   {
+    if (id == TRX_SYS_SPACE && recv_sys.check_sys_truncate())
+      goto clear;
     bool success;
     while (fil_space_extend_must_retry(this, node, desired_size, &success))
       mysql_mutex_lock(&fil_system.mutex);
@@ -1103,8 +1131,10 @@ bool fil_assign_new_space_id(uint32_t *space_id)
 }
 
 /** Read the first page of a data file.
+@param dpage   copy of a first page, from the doublewrite buffer, or nullptr
+@param no_lsn  whether to skip the FIL_PAGE_LSN check
 @return whether the page was found valid */
-bool fil_space_t::read_page0()
+bool fil_space_t::read_page0(const byte *dpage, bool no_lsn) noexcept
 {
   ut_ad(fil_system.is_initialised());
   mysql_mutex_assert_owner(&fil_system.mutex);
@@ -1121,32 +1151,26 @@ bool fil_space_t::read_page0()
     ut_ad("this should not happen" == 0);
     return false;
   }
-  const bool ok= node->is_open() || fil_node_open_file(node);
+  const bool ok= node->is_open() || fil_node_open_file(node, dpage, no_lsn);
   release();
   return ok;
 }
 
-/** Look up a tablespace and ensure that its first page has been validated. */
-static fil_space_t *fil_space_get_space(uint32_t id)
-{
-  if (fil_space_t *space= fil_space_get_by_id(id))
-    if (space->read_page0())
-      return space;
-  return nullptr;
-}
-
 void fil_space_set_recv_size_and_flags(uint32_t id, uint32_t size,
                                        uint32_t flags)
 {
   ut_ad(id < SRV_SPACE_ID_UPPER_BOUND);
+  mysql_mutex_assert_owner(&recv_sys.mutex);
   mysql_mutex_lock(&fil_system.mutex);
-  if (fil_space_t *space= fil_space_get_space(id))
-  {
-    if (size)
-      space->recv_size= size;
-    if (flags != FSP_FLAGS_FCRC32_MASK_MARKER)
-      space->flags= flags;
-  }
+  if (fil_space_t *space= fil_space_get_by_id(id))
+    if (space->read_page0(recv_sys.dblwr.find_page(page_id_t(id, 0), LSN_MAX),
+                          true))
+    {
+      if (size)
+        space->recv_size= size;
+      if (flags != FSP_FLAGS_FCRC32_MASK_MARKER)
+        space->flags= flags;
+    }
   mysql_mutex_unlock(&fil_system.mutex);
 }
 
@@ -1161,12 +1185,16 @@ bool fil_space_t::open(bool create_new_db)
   bool success= true;
   bool skip_read= create_new_db;
 
+  const page_t *page= skip_read
+    ? nullptr
+    : recv_sys.dblwr.find_page(page_id_t{id, 0}, LSN_MAX);
+
   mysql_mutex_lock(&fil_system.mutex);
 
   for (fil_node_t *node= UT_LIST_GET_FIRST(chain); node;
        node= UT_LIST_GET_NEXT(chain, node))
   {
-    if (!node->is_open() && !fil_node_open_file_low(node))
+    if (!node->is_open() && !fil_node_open_file_low(node, page, page))
     {
 err_exit:
       success= false;
@@ -1184,7 +1212,7 @@ err_exit:
       continue;
     }
 
-    if (!node->read_page0())
+    if (!node->read_page0(page, true))
     {
       fil_system.n_open--;
       os_file_close(node->handle);
@@ -1193,6 +1221,7 @@ err_exit:
     }
 
     skip_read= true;
+    page= nullptr;
   }
 
   if (!create_new_db)
@@ -1596,17 +1625,18 @@ inline void mtr_t::log_file_op(mfile_type_t type, uint32_t space_id,
   m_last= nullptr;
 
   const size_t len= strlen(path);
-  const size_t new_len= type == FILE_RENAME ? 1 + strlen(new_path) : 0;
+  const size_t new_len= new_path ? 1 + strlen(new_path) : 0;
   ut_ad(len > 0);
   byte *const log_ptr= m_log.open(1 + 3/*length*/ + 5/*space_id*/ +
                                   1/*page_no=0*/);
+  *log_ptr= type;
   byte *end= log_ptr + 1;
   end= mlog_encode_varint(end, space_id);
   *end++= 0;
-  if (UNIV_LIKELY(end + len + new_len >= &log_ptr[16]))
+  const byte *const final_end= end + len + new_len;
+  if (UNIV_LIKELY(final_end >= &log_ptr[16]))
   {
-    *log_ptr= type;
-    size_t total_len= len + new_len + end - log_ptr - 15;
+    size_t total_len= final_end - log_ptr - 15;
     if (total_len >= MIN_3BYTE)
       total_len+= 2;
     else if (total_len >= MIN_2BYTE)
@@ -1617,13 +1647,13 @@ inline void mtr_t::log_file_op(mfile_type_t type, uint32_t space_id,
   }
   else
   {
-    *log_ptr= static_cast<byte>(type | (end + len + new_len - &log_ptr[1]));
+    *log_ptr= static_cast<byte>(*log_ptr | (final_end - &log_ptr[1]));
     ut_ad(*log_ptr & 15);
   }
 
   m_log.close(end);
 
-  if (type == FILE_RENAME)
+  if (new_path)
   {
     ut_ad(strchr(new_path, '/'));
     m_log.push(reinterpret_cast<const byte*>(path), uint32_t(len + 1));
@@ -2885,12 +2915,12 @@ fil_io_t fil_space_t::io(const IORequest &type, os_offset_t offset, size_t len,
 	}
 
 	DBUG_EXECUTE_IF("intermittent_recovery_failure",
-			if (type.is_read() && !(~get_rnd_value() & 0x3ff0))
+			if (type.is_read() && !(~my_timer_cycles() & 0x3ff0))
 			goto io_error;);
 
 	DBUG_EXECUTE_IF("intermittent_read_failure",
 			if (srv_was_started && type.is_read() &&
-			    !(~get_rnd_value() & 0x3ff0)) goto io_error;);
+			    !(~my_timer_cycles() & 0x3ff0)) goto io_error;);
 
 	if (UNIV_LIKELY_NULL(UT_LIST_GET_NEXT(chain, node))) {
 		ut_ad(this == fil_system.sys_space
@@ -3000,6 +3030,7 @@ void IORequest::read_complete(int io_error) const
   {
     sql_print_error("InnoDB: Read error %d of page " UINT32PF " in file %s",
                     io_error, id.page_no(), node->name);
+    recv_sys.free_corrupted_page(id, *node);
     buf_pool.corrupted_evict(bpage, buf_page_t::READ_FIX);
   corrupted:
     if (recv_recovery_is_on() && !srv_force_recovery)
@@ -3009,13 +3040,8 @@ void IORequest::read_complete(int io_error) const
       mysql_mutex_unlock(&recv_sys.mutex);
     }
   }
-  else if (dberr_t err= bpage->read_complete(*node))
-  {
-    if (err != DB_FAIL)
-      ib::error() << "Failed to read page " << id.page_no()
-                  << " from file '" << node->name << "': " << err;
+  else if (bpage->read_complete(*node))
     goto corrupted;
-  }
 
   node->space->release();
 }
