@@ -185,13 +185,6 @@ lock_update_split_left(
 void lock_update_merge_left(const buf_block_t& left, const rec_t *orig_pred,
                             const page_id_t right);
 
-/** Update the locks when a page is split and merged to two pages,
-in defragmentation. */
-void lock_update_split_and_merge(
-	const buf_block_t* left_block,	/*!< in: left page to which merged */
-	const rec_t* orig_pred,		/*!< in: original predecessor of
-					supremum on the left page before merge*/
-	const buf_block_t* right_block);/*!< in: right page from which merged */
 /*************************************************************//**
 Resets the original locks on heir and replaces them with gap type locks
 inherited from rec. */
@@ -820,15 +813,6 @@ public:
   bool is_writer() const { return latch.have_wr(); }
   /** @return whether the current thread is holding lock_sys.latch */
   bool is_holder() const { return latch.have_any(); }
-  /** Returns whether a cell of hash table where the lock is stored is latched.
-  @param lock the lock which cell is checked
-  @return whether the lock's cell is latched */
-  bool is_cell_locked(const lock_t &lock) {
-    return hash_table::latch(
-               hash_get(lock.type_mode)
-                   .cell_get(lock.un_member.rec_lock.page_id.fold()))
-        ->is_locked();
-  }
   /** Assert that a lock shard is exclusively latched (by some thread) */
   void assert_locked(const lock_t &lock) const;
   /** Assert that a table lock shard is exclusively latched by this thread */
