@@ -196,13 +196,11 @@ row_vers_impl_x_locked_low(
 			version, clust_index, clust_offsets,
 			heap, &prev_version, mtr, 0, NULL,
 			dict_index_has_virtual(index) ? &vrow : NULL);
-		ut_d(bool owns_trx_mutex = trx->mutex_is_owner());
-		ut_d(if (!owns_trx_mutex)
-		    trx->mutex_lock();)
+
+		ut_d(trx->mutex_lock());
 		const bool committed = trx_state_eq(
 			trx, TRX_STATE_COMMITTED_IN_MEMORY);
-		ut_d(if (!owns_trx_mutex)
-		  trx->mutex_unlock();)
+		ut_d(trx->mutex_unlock());
 
 		/* The oldest visible clustered index version must not be
 		delete-marked, because we never start a transaction by
@@ -330,7 +328,7 @@ not_locked:
 
 		/* We check if entry and rec are identified in the alphabetical
 		ordering */
-		if (0 == cmp_dtuple_rec(entry, rec, offsets)) {
+		if (0 == cmp_dtuple_rec(entry, rec, index, offsets)) {
 			/* The delete marks of rec and prev_version should be
 			equal for rec to be in the state required by
 			prev_version */
@@ -348,7 +346,7 @@ not_locked:
 			dtuple_set_types_binary(
 				entry, dtuple_get_n_fields(entry));
 
-			if (0 != cmp_dtuple_rec(entry, rec, offsets)) {
+			if (cmp_dtuple_rec(entry, rec, index, offsets)) {
 
 				break;
 			}
@@ -402,8 +400,7 @@ row_vers_impl_x_locked(
 	const rec_t*	clust_rec;
 	dict_index_t*	clust_index;
 
-	/* The current function can be called from lock_rec_unlock_unmodified()
-	under lock_sys.wr_lock() */
+	lock_sys.assert_unlocked();
 
 	mtr_start(&mtr);
 
